function [path,descriptif]=transcription(C,mode,K,harmoniques,distance,L,lissage)
%   [path,descriptif]=transcription(C,mode,K,harmoniques,distance,L,lissage)
%   This function computes the chord transcription from the chromagram
%
%   Inputs :     
%       * C : chromagram
%       * mode : decoding method
%                  - 0 (default) : all chords mapped to major and minor
%                  - 1 : no mapping
%       * K : types of chords 
%       * harmoniques : number of harmonics (1,4 or 6)
%       * distance : distances (euc, is1, is2, kl1 or kl2)
%       * L : neighbourhood sizes (odd number)
%       * lissage : filtering methods
%                  - 1 : low pass
%                  - 2 : median
%   Output :
%       * path : chord transcriptions
%       * descriptif : parameters used for the transcriptions
%
%   Main reference (available at http://www.tsi.enst.fr/~oudre/):
%   Oudre L., Grenier Y., Fvotte C., "Chord recognition by fitting 
%   rescaled chroma vectors to chord templates", 2009.
%
%   Copyright (C) July 2009  L.OUDRE
%   Inquiries, bug report: laurent.oudre@telecom-paristech.fr
%
%   This program is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
%   USA
                
%% Default parameters

if nargin<7
    lissage=1;
end

if nargin<6
    L=1;
end

if nargin<5
    distance{1}='euc';
end

if nargin<4
    harmoniques=1;
end

if nargin<3
    K={'maj','min'};
end

if nargin<2
    mode=0;
end

L=(L-1)/2;
[F,N]=size(C);

%% Compute the number of simulations

N_harmoniques=length(harmoniques);
N_distance=length(distance);
N_L=length(L);
N_lissage=length(lissage);

N_simu=N_harmoniques*N_distance*N_L*N_lissage;

path=zeros(N,N_simu); % matrix containing all the chord transcriptions



for i_harmoniques=1:N_harmoniques

%% Step 1 : Compute the chord dictionnaries

    N_K=length(K);
    W=zeros(12,12*N_K);

    for i_K=1:N_K
        pattern=find_pattern(K{i_K});
        pattern=create_harmonique(pattern,harmoniques(i_harmoniques));
        W(:,(i_K-1)*12+1:i_K*12)=create_pattern(pattern);
    end
    
    W=W+eps;
    W=norm1c(W);

  
    for i_distance=1:N_distance
        
%% Step 2 : Compute the distance matrix

        D=zeros(12*N_K,N);

        for n=1:N
            D(:,n)=compute_distance(C(:,n),W,distance{i_distance})';
        end
        
        silences=find(isnan(sum(D)));
        D(find(isnan(D)))=0;       

        for i_lissage=1:N_lissage
 
             for i_L=1:N_L

%% Step 3 : Filtering

                [critere]=lissage_L(D,lissage(i_lissage),L(i_L)); 
                [garb,idx]=min(critere);
           
                indice=i_L+(i_lissage-1)*N_L+(i_distance-1)*N_L*N_lissage+(i_harmoniques-1)*N_L*N_lissage*N_distance;
                descriptif{indice,1}={['harmoniques=' num2str(harmoniques(i_harmoniques)) ' - distance=' distance{i_distance} ' - lissage=' num2str(lissage(i_lissage)) ' - L=' num2str(2*L(i_L)+1)]};
          
                path(:,indice)=idx(:);               
                
           
            end
        end
    end
end



switch mode
    
    case 0

    %Mapping of all chord types to major and minor
    for i_K=1:N_K
    [pattern,quality]=find_pattern(K{i_K});
    path((path>=12*(i_K-1)+1)&(path<=i_K*12))=mod(path((path>=12*(i_K-1)+1)&(path<=i_K*12))-1,12)+quality*12;
    end

    otherwise
        
        path=path-1;

end
        
path(silences,:)=24;







function [pattern,quality]=find_pattern(chord)

%%% This function returns the binary model of a chord type
     
pattern=zeros(12,1);

mode_vect=[0 1 0 0 0];

switch chord

    case 'maj'
        pattern([1 5 8])=1; 
        quality = 0;
    case 'min'
        pattern([1 4 8])=1;
        quality = 1;        
    case 'dim' 
        pattern([1 4 7])=1;
        quality = 2;    
    case 'aug' 
        pattern([1 5 9])=1;
        quality = 3;
    case 'maj7' 
        pattern([1 5 8 12])=1;
        quality = 0;
    case 'min7' 
        pattern([1 4 8 11])=1;
        quality = 1;
    case '7' 
        pattern([1 5 8 11])=1;
        quality = 0;        
    case 'dim7' 
        pattern([1 4 7 10])=1;
        quality = 2;
    case 'hdim7'
        pattern([1 4 7 11])=1;
        quality = 2;
    case 'minmaj7'
        pattern([1 4 8 12])=1;
        quality = 1;     
    case 'maj6'
        pattern([1 5 8 10])=1;
        quality = 0;        
    case 'min6' 
        pattern([1 4 8 10])=1;
        quality = 1;
    case '9'
        pattern([1 5 8 11 3])=1;
        quality = 0;          
    case 'maj9'
        pattern([1 5 8 12 3])=1;
        quality = 0;    
    case 'min9'
        pattern([1 4 8 11 2])=1;
        quality = 1;    
    case 'sus4'
        pattern([1 6 8])=1;
        quality = 4;
    case 'sus2'
        pattern([1 3 8])=1;
        quality = 4;

    otherwise
        error('Pas bon type d''accord')
end

quality=mode_vect(quality+1);











function [d]=compute_distance(c_n,W,distance)
% % % This function calculates the distances between a chromagram frame and
% % % a chord dictionnary
% % % Input :
% % %     * c_n : chromagram frame
% % %     * W : chord dictionnary
% % %     * distance : distance to compute
% % % Ouput :
% % %     * d : distances between the chromagram frame and the chord
% % % dictionnary


[M,K]=size(W);
C_n=repmat(c_n,1,K);

switch lower(distance)
    case 'is1'
        d=M*log(mean(C_n./W))-sum(log(C_n./W));
    case 'is2'
        d=M*log(mean(W./C_n))-sum(log(W./C_n));
    case 'euc'
        d=sqrt(sum(W.^2)-(sum(C_n.*W).^2)./sum(C_n.^2));
    case 'kl1'
        C_n=repmat(c_n/sum(c_n),1,K);
        d=1-exp(-sum(C_n.*log(C_n./W)));
    case 'kl2'
        C_n=repmat(c_n/sum(c_n),1,K);
        d=sum(W.*log(W./C_n)-W+C_n);    
     
    otherwise
        error('Unknown distance');
end




function B=create_pattern(pattern)
%%% This function computes the chord dictionnary from the original chord
%%% pattern
    
    B=zeros(12, length(pattern));
    B(:,1)=pattern;
    for i=2:12
        b=[B(end,i-1) ; B(:,i-1)];
        B(:,i)=b(1:12);
    end
    

function pattern_out=create_harmonique(pattern,nbharmo)
%%% This function computes the pattern taking account harmonics from the
%%% original binary chord pattern
h=0.6;
harmonique = [0 0 7 0 4 7];
poids=h.^(0:5);    

harmonique=harmonique(1:nbharmo);
poids=poids(1:nbharmo);

vector=find(pattern)-1;
N=length(vector);

pattern_out=zeros(12,1);

for k=1:N
    for i=1:12
        pattern_out(i)=pattern_out(i)+sum(poids(find(mod(harmonique+vector(k),12)+1==i)));
    end
end




function [critere]=lissage_L(D,lissage,L)
%%% This function calculates the filtered version of the matrix of
%%% distances
    
    
[N1,N2]=size(D);
critere=zeros(N1,N2);

switch lissage
    case 1
        %% Low-pass filtering
        if L>0

            D2=[zeros(N1,L) D zeros(N1,L)];
            for n=1:N2
                   critere(:,n)=mean(D2(:,n:n+2*L),2);
            end

        else
            critere=D;
        end

    case 2
        %% Median filtering
        if L>0
            critere=medfilt1(D,2*L+1,N2,2);
        else
            critere=D;
        end

    otherwise
        error('Unknown filtering method');
        

end




function n = norm1c(m)
%%% This function normalizes a matrix

if nargin < 1,error('Not enough input arguments.'); end

[mr,mc] = size(m);
if (mr == 1)
  n = ones(1,mc);
else
  n =ones(mr,1)*(ones./sum(m)).*m;
end




